//
// Title       : Homework 6, register file test fixture
// Author      : Carl Ebeling
// Company     : CSE 370
//
//-----------------------------------------------------------------
//
// File        : regfile_tf.v
//				 Homework #7
// Description : This test fixture is used to test the register file
//
//-----------------------------------------------------------------
`timescale 1ns / 1ns


module regfile_tf (CLK, reset, AddrA, AddrB, AddrW, Write, opcode, result);
 
   input CLK, reset;
   output [1:0] AddrA, AddrB, AddrW; // Register file addresses 
   output [2:0] opcode;		// ALU opcode
   output 	Write;		// Write to register file
   input [15:0] result;		// ALU output for checking results
   
   reg [1:0] 	AddrA, AddrB, AddrW; // Register file addresses
   reg [2:0] 	opcode;		// ALU opcode
   reg 		Write;		// Write to register file

   integer 	i, errors;
   parameter 	AAA = 0, SUB = 6, ZZZ = 2, XXX = 3, XOR = 1, XNOR = 5;

   initial begin 
      errors = 0;
      i = 0;
      Write = 0;
   end

   always @(posedge CLK) begin
      if (!reset) begin
	 case (i)
	   0: begin
	      Write = 1;
	      AddrA = 1;		// 0
	      AddrB = 2;		// 0
	      AddrW = 0;		// 0
	      opcode = AAA; 
	   end
	   1: begin
	      if (result !== 0) begin
		 $display("***Error***(1) ALU result should be 0");
		 errors = errors + 1;
		 $stop;
	      end
	      AddrA = 0;		// 0
	      AddrB = 3;		// 0
	      AddrW = 1;		// 1
	      opcode = ZZZ; 
	   end
	   2: begin
	      if (result !== 1) begin
		 $display("***Error***(2) ALU result should be 1");
		 errors = errors + 1;
		 $stop;
	      end
	      AddrA = 1;		 // 1
	      AddrB = 2;		 // 0
	      AddrW = 2;		 // 2
	      opcode = ZZZ; 
	      Write = 0;
	   end
	   3: begin
	      if (result !== 2) begin
		 $display("***Error***(2) ALU result should be 2");
		 errors = errors + 1;
		 $stop;
	      end
	      AddrA = 1;		 // 1
	      AddrB = 2;		 // 0
	      AddrW = 2;		 // 2
	      opcode = ZZZ; 
	      Write = 1;
	   end
	   4: begin
	      if (result !== 2) begin
		 $display("***Error***(3) ALU result should be 2");
		 errors = errors + 1;
		 $stop;
	      end
	      AddrA = 2;		// 2
	      AddrB = 2;		// 2
	      AddrW = 0;		// 4
	      opcode = AAA; 
	   end
	   5: begin
	      if (result !== 4) begin
		 $display("***Error***(4) ALU result should be 4");
		 errors = errors + 1;
		 $stop;
	      end
	      AddrA = 2;		// 2
	      AddrB = 0;		// 4
	      AddrW = 3;		// 2
	      opcode = SUB; 	// B-A
	   end
	   6: begin
	      if (result !== 2) begin
		 $display("***Error***(5) ALU result should be 2");
		 errors = errors + 1;
		 $stop;
	      end
	      AddrA = 0;		// 4
	      AddrB = 3;		// 2
	      AddrW = 3;		// 4
	      opcode = XXX; 
	   end
	   7: begin
	      if (result != 4) begin
		 $display("***Error***(6) ALU result should be 4");
		 errors = errors + 1;
		 $stop;
	      end
	      AddrA = 3;		 // 4
	      AddrB = 1;		 // 1
	      AddrW = 2;		 // -3
	      opcode = SUB; 
	   end
	   8: begin
	      if (result !== 16'b1111111111111101) begin
		 $display("***Error***(7) ALU result should be -3");
		 errors = errors + 1;
		 $stop;
	      end
	      AddrA = 2;		 // -3
	      AddrB = 3;		 // 4
	      AddrW = 0;		 // -7
	      opcode = XOR; 
	   end
	   9: begin
	      if (result !== 16'b1111111111111001) begin
		 $display("***Error***(8) ALU result should be -7");
		 errors = errors + 1;
		 $stop;
	      end
	      AddrA = 0;		  // -7
	      AddrB = 1;		  // 1
	      AddrW = 1;		  // 8
	      opcode = SUB; 	  // 1 - -7
	   end
	   10: begin
	      if (result !== 8) begin
		 $display("***Error***(9) ALU result should be 8");
		 errors = errors + 1;
		 $stop;
	      end
	      AddrA = 1;		  // 8	= 1000
	      AddrB = 2;		  // -3 = 1101
	      AddrW = 1;		  // 10	= 1010
	      opcode = XNOR; 
	   end
	   11: begin
	      if (result !== 10) begin
		 $display("***Error***(10) ALU result should be 14");
		 errors = errors + 1;
		 $stop;
	      end
	   end
	   12: begin
	      if (errors == 0) 
		$display("Test finished successfully");	 
	      else
		$display("*****%d Errors found*****", errors);
	      $finish;
	   end
	   
	 endcase	
	 i = i + 1;
      end
   end
   
endmodule
